//=========================================================================
//		NSWindow+WindowContentSize.m					TomatoTorrent
//=========================================================================
//  Created by Sarwat Khan on Wed Jun 11 2003.
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khans name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import "NSWindow+WindowContentSize.h"

@implementation NSWindow (WindowContentSize)
- setMinimumContentSize:(NSSize)size maximumSize:(NSSize)max
{
	NSRect		frame = [self frame];
	NSRect		contentFrame = [[self contentView] frame];
	NSSize		contentSize = contentFrame.size;
	NSSize		delta;
	
	if (size.width == 0) size.width = FLT_MAX;
	if (size.height == 0) size.height = FLT_MAX;
	if (max.width == 0) max.width = FLT_MAX;
	if (max.height == 0) max.height = FLT_MAX;
	
	delta.width = frame.size.width - contentSize.width;
	delta.height = frame.size.height - contentSize.height;
	
	size.width += delta.width; max.width += delta.width;
	size.height += delta.height; max.height += delta.height;
	
	[self setMinSize:size];
	[self setMaxSize:max];
	
	[self setShowsResizeIndicator:NSEqualSizes(size, max) == NO];
	
	return self;
}

- setContentSize:(NSSize)size display:(BOOL)display animate:(BOOL)animate;
{
	NSRect		windowFrame, temp;
	NSSize		currentSize;
	float		origWindowHeight,
				origWindowWidth;
	
	windowFrame = [self frame];
	temp = [[self contentView] frame];
	currentSize = temp.size;

	origWindowHeight = windowFrame.size.height;
	windowFrame.size.height -= currentSize.height;
	windowFrame.size.height += size.height;
	windowFrame.origin.y += origWindowHeight - windowFrame.size.height;

	origWindowWidth = currentSize.width;
	windowFrame.size.width -= currentSize.width;
	windowFrame.size.width += size.width;
	windowFrame.origin.x += (origWindowWidth - windowFrame.size.width)/2;
	
	[self setFrame:windowFrame display:display animate:animate];
	
	return self;
}

- switchContentViewTo:view showingViewDuringResize:empty display:(BOOL)display animate:(BOOL)animate;
{
	NSRect		frame = [view frame];
	NSSize		newSize = frame.size;
	id			oldView = [self contentView];
	NSRect		oldFrame = [oldView frame];
	
	if (oldView != empty)
		[self setContentView:empty];
	
	[self setContentSize:newSize display:display animate:animate];
	
	if (empty != view)
	{
		[self setContentView:view];
		[oldView setFrame:oldFrame];
	}
	
	return self;
}

@end
