//=========================================================================
//		NSFileManager+LocalizedPaths.m					BitTorrent
//=========================================================================
//  Created by Sarwat Khan on Wed Jun 18 2003.
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khans name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import "NSFileManager+LocalizedPaths.h"

@implementation NSFileManager (LocalizedPaths)
- (NSString*)displayPathForPath:(NSString*)path;
{
	id		standardizedPath,
			components,
			oldComponents, newComponents,
			ite, object;
	BOOL	canUseAlt;
	
	canUseAlt = [self respondsToSelector:@selector(componentsToDisplayForPath:)];
	standardizedPath = [path stringByStandardizingPath];
	
		//the below algo isn't good with non-tilde paths, specifically when dealing with
		//paths that belong to another volume. For non-~ paths componentsToDisplayForPath: 
		//is used if possible.
	
	if ([standardizedPath isEqual:[standardizedPath stringByAbbreviatingWithTildeInPath]] == NO
		|| canUseAlt == NO)
	{
		components = [[path stringByAbbreviatingWithTildeInPath] pathComponents];
			
		newComponents = [NSMutableArray array];
		oldComponents = [NSMutableArray array];
		
		ite = [components objectEnumerator];
		
		while (object = [ite nextObject])
		{
			[oldComponents addObject:object];
			
			//I'm sorry if this is confusing. Really, I am.
			
			if (!([object isEqual:@"~"] == YES && [newComponents count] == 0))		//don't expand ~ into the user name.
				[newComponents addObject:
					[self displayNameAtPath:
						[[NSString pathWithComponents:oldComponents] stringByExpandingTildeInPath]]];
			else
				[newComponents addObject:object];
		}
	}
	else
	{
		newComponents = [self componentsToDisplayForPath:standardizedPath];
	}
	
	return [NSString pathWithComponents:newComponents];
}

@end
