//==============================================================================
//		SKDataFormatters.m					BitTorrent
//==============================================================================
//  Created by Sarwat Khan on Sun Jun 15 2003.
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khan’s name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import <Foundation/Foundation.h>

#import "SKDataFormatters.h"
#import "SKLocalizedString.h"
#import "NSFileManager+LocalizedPaths.h"

#define kKilobyte       1024.0
#define kMegabyte       (1024 * kKilobyte)
#define kGigabyte       (1024 * kMegabyte)


@implementation SKLocalizedStringFormatter
- (void) dealloc
{
	[_table release];
	[super dealloc];
}

- setBundle:bundle;
{
	_bundle = bundle;
	return self;
}

- setTableName:tableName;
{
	_table = [tableName copy];
	return self;
}

- (NSString *)stringForObjectValue:(id)obj;
{
	if (_table && _bundle)
		return SKLocalizedStringFromTableInBundle(obj, _table, _bundle, nil);
	return SKLocalizedString(obj, nil);
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error
{
	return NO;
}
@end

@implementation SKBooleanFormatter
- (NSString *)stringForObjectValue:(id)obj;
{
	id	result = nil;
	
	if ([obj respondsToSelector:@selector(intValue)])
	{
		if ([obj intValue])
		{
			result = SKLocalizedString(@"Yes", @"Yes - boolean truth value");
		}
		else
		{
			result = SKLocalizedString(@"No", @"No - boolean truth value");
		}
	}
	
	if (result == nil) result = [obj description];
	
	return result;
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error
{
	return NO;	//+++ it would be useful to implement this
}
@end

@implementation SKUnixTimeFormatter
- init
{
	return [super initWithDateFormat:SKLocalizedString(
										@"%a %b %e '%y @ %I:%M:%S %p", 
										@"date format (unix time formatter)")
				allowNaturalLanguage:NO];
}

- (NSString *)stringForObjectValue:(id)obj;
{
	id		result = nil,
			date;
	
	if ([obj respondsToSelector:@selector(doubleValue)])
	{
		date = [[NSDate alloc] initWithTimeIntervalSinceReferenceDate:
			[obj doubleValue] - NSTimeIntervalSince1970];
			
		result = [super stringForObjectValue:date];//[[[date description] retain] autorelease];
		
		[date release];
	}
	
	if (result == nil)
		result = [obj description];
	
	return result;
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error
{
	return NO;
}
@end

@implementation SKDataSizeFormatter
- formatMegabyteValues
{
	_factor = 1024 * 1024;
	return self;
}

- (NSString *)stringForObjectValue:(id)object;
{
	id		result, format;
	double	double_size;
	
	result = object;
	
	if (object == nil)
	{
		result = SKLocalizedString(@"N/A", nil);
	}
	else if ([object respondsToSelector:@selector(doubleValue)])
	{
		double_size = [object doubleValue];
		
		if (_factor) double_size *= _factor;
		
		if (double_size < kKilobyte)
			format 		= SKLocalizedString(@"%.0f bytes", nil);
			
		else if (double_size < kMegabyte)
			double_size = double_size / kKilobyte, 
			format 		= SKLocalizedString(@"%.1f K", nil);
			
		else if (double_size < kGigabyte)
			double_size = double_size / kMegabyte, 
			format 		= SKLocalizedString(@"%.1f MB", nil);
			
		else
			double_size = double_size / kGigabyte, 
			format 		= SKLocalizedString(@"%.2f GB", nil);
	
		result = [NSString stringWithFormat:format, double_size];
	}
	
	return result;
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error
{
	return NO;
}
@end

@implementation SKFilePathFormatter
- (NSString *)stringForObjectValue:(id)object;
{
	id		result;
	
	result = object;
	
	if ([object isKindOfClass:[NSString class]])
		result = [[NSFileManager defaultManager] displayPathForPath:object];
		
	return result;
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error
{
	return NO;
}
@end


@implementation SKFractionFormatter
- setMultiplierWithDouble:(double)scale;		//integer of denominator
{
	_scale = scale;
	return self;
}

- (NSString *)stringForObjectValue:(id)object;
{
	id		result;
	id		formatObj = object;
	
	if (_scale > 0 && [object respondsToSelector:@selector(doubleValue)])
		formatObj = [[NSNumber allocWithZone:[self zone]] initWithDouble:[object doubleValue] * _scale];
	
	result = [super stringForObjectValue:formatObj];
	
	if (formatObj != object)
		[formatObj release];
	
	return result;
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error
{
	return NO;	//really.
}

@end
