//=========================================================================
//		SKInternetConfig.m
//=========================================================================
//  Created by Sarwat Khan on Wed Jun 11 2003.
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khans name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//	This is inspired by OmniAppKit code (omnigroup.com).

#import "SKInternetConfig.h"
#import "NSBundle+MainBundleExtras.h"

@implementation SKInternetConfig

+ instance
{
	static id		this = nil;
	
	if (this == nil) this = [self new];
	
	return this;
}

- init
{
	[super init];

	if (ICStart(&_component, [NSBundle applicationSignature]) != noErr)
	{
		[self autorelease];
        self = nil;
	}
    
	return self;
}

- (void) dealloc
{
    ICStop(_component);	
	[super dealloc];
}

- (NSString *)downloadFolderPath;
{
    ICAttr 			attribute;
    Size 			size;
    ICFileSpecPtr	icfileSpecPtr = NULL;
    AliasPtr 		aliasPtr;
    Boolean	 		aliasUpdated;
    FSRef 			downloadFolderRef;
	bool			haveValidRef = false;
    uint8_t			path[PATH_MAX];
	OSErr			error;
	id				result = nil;

    if (ICGetPref(_component, kICDownloadFolder, &attribute, NULL, &size) == noErr)
	{
		icfileSpecPtr = malloc(size);
		if (ICGetPref(_component, kICDownloadFolder, &attribute, icfileSpecPtr, &size) == noErr)
		{
			aliasPtr = &icfileSpecPtr->alias;
			error = FSResolveAlias(NULL, &aliasPtr, &downloadFolderRef, &aliasUpdated);
			if (error == noErr)
			{
				haveValidRef = true;
			}
			else
			{
				error = FSpMakeFSRef(&icfileSpecPtr->fss, &downloadFolderRef);
				haveValidRef = (error == noErr);
			}
			
			if (haveValidRef && FSRefMakePath(&downloadFolderRef, path, PATH_MAX) == noErr)
			{
				result = [NSString stringWithUTF8String:(const char*)path];
			}
			
		}
	}
	
	if (icfileSpecPtr) free(icfileSpecPtr);
	
	return result;
}


@end
