//==============================================================================
//		SKRange.m					TomatoTorrent
//==============================================================================
//  Created by Sarwat Khan on Fri Jun 13 2003.
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khans name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import "SKRange.h"


@implementation SKRange

- initWithRange:(NSRange)range;
{
	return [self initStartingAt:range.location maximum:NSMaxRange(range)];
}

- initStartingAt:(int)start maximum:(int)stop 
{
	return [self initStartingAt:start maximum:stop by:(start < stop ? 1 : -1)];
}

- initStartingAt:(int)start maximum:(int)stop by:(int)step;
{
	[super init];
	
	_start = start;
	_max = stop;
	_step = step;
	
	return self;
}

- (void) dealloc
{
	[_current release];
	[super dealloc];
}

- allObjects;
{
	NSZone	*zone;
	int		i, sign, 
			stop = _max;
	id		number, temp, result;

	zone = [self zone];
	temp = [[NSMutableArray allocWithZone:zone] init];
	
		sign = 1;
		if (_step < 0) 
		{
			sign = -1;
			stop = -stop;
		}
		
		i = _start;
		while (i * sign < stop)
		{
			number = [[NSNumber allocWithZone:zone] initWithInt:i];
			[temp addObject:number];
			[number release];
			i += _step;
		}
		result = [temp copyWithZone:zone];
	[temp release];
	
	return [result autorelease];
}

- nextObject
{
	int		value;
	
	value = _start;

	if (_current) 
	{
		value = [_current intValue];
		if (!_autorelease)
			[_current release];
		else
			[_current autorelease];
		
		_current = nil;
		
		value += _step;
	}

	if (_step > 0)
	{
		if (value < _max)
			_current = [[NSNumber allocWithZone:[self zone]] initWithInt:value];
	}
	else
	{
		if (_max < value)
			_current = [[NSNumber allocWithZone:[self zone]] initWithInt:value];		
	}
	
	return _current;
}

- setAutoreleasesObjects:(BOOL)autorelease
{
	_autorelease = autorelease;
	return self;
}

- (BOOL) autoreleasesObjects
{
	return _autorelease;
}

@end

@implementation NSNumber (SKRange)
- to:number;
{
	return [self toInt:[number intValue]];
}

- to:number by:step;
{
	return [self toInt:[number intValue] byInt:[step intValue]];
}

- toInt:(int)number;
{
	return [self toInt:number byInt:([self intValue] < number ? 1 : -1)];
}

- toInt:(int)number byInt:(int)step;
{
	return [[[SKRange allocWithZone:[self zone]] initStartingAt:[self intValue] maximum:number by:step] autorelease];
}

- doFunction:(id (*)(id, void*))function context:(void*)context;
{
	return function(self, context);
}

@end

@implementation NSArray (SKRange)
- (NSEnumerator*)keyEnumerator;
{
	id		result;
	
	result = [[SKRange allocWithZone:[self zone]] initWithRange:NSMakeRange(0, [self count])];
	
	return [result autorelease];
}
@end

@implementation NSSet (SKRange)
- (NSEnumerator*)keyEnumerator;
{
	id		result;
	
	result = [[SKRange allocWithZone:[self zone]] initWithRange:NSMakeRange(0, [self count])];
	
	return [result autorelease];
}
@end

