//==============================================================================
//		ModuleOptionsController.m					TomatoTorrent
//==============================================================================
//  Created by Sarwat Khan on Thu Jun 12 2003.
//  Copyright (c) 2003 Sarwat Khan. All rights reserved.

#import "ModuleOptionsController.h"

#import "TomatoTorrentAgent.h"
#import "AppDelegate.h"
#import "SKDataFormatters.h"
#import "NSDictionary+DataTransform.h"

@interface NSValue (LazySarwat)
- (BOOL) isIntegerValue;
@end

@interface ModuleOptionsValueTableColumn: NSTableColumn
{
	id	_checkboxCell;
	id	_list;
	SEL	_sel;
	id	_defaults;
}
- setOptionsList:list defaultsSelector:(SEL)sel;
@end;

@interface ModuleOptionsController (Private)
- _setOptionsList:list;
- _defaultOptions;
@end

@interface ModuleOptionsController (NSTableDataSource)
- _nameOptionsDataWithKey:key;
- _valueOptionsDataWithKey:key;
- _descOptionsDataWithKey:key;
@end

@interface ModuleOptionsController (NSTableViewDelegate)
- (void)tableViewSelectionDidChange:(NSNotification *)notification;
@end

@implementation ModuleOptionsController

- init
{
	[super init];
	
	_intFormatter = [NSNumberFormatter new];
	[_intFormatter setFormat:@"0"];
	[_intFormatter setAllowsFloats:NO];
	
	_realFormatter = [NSNumberFormatter new];
	[_realFormatter setFormat:@"0.0######"];
	
	return self;
}

- (void) dealloc
{
	[_data release];
	
	[_intFormatter release];
	[_realFormatter release];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	[super dealloc];
}

- (void) awakeFromNib
{
	[_table sizeLastColumnToFit];
	[_table selectRow:0 byExtendingSelection:NO];
}

//------------------------------------------------------------------------------
//		 setModuleOptions:defaultsSelector:
//------------------------------------------------------------------------------
//	can be called multiple times, only first time has effect. this is so defaults
//	aren't loaded until needed.
//
//	Hacked in support for working without a nib, so formatterForKey's logic is public.
//
//	

- setModuleOptions:(NSMutableDictionary*)options defaultsSelector:(SEL)defaultsSelector
	controllingTableForWindow:window	//may be nil, in which case _table's window 
										//must be defined. It's not defined when the
										//table's are not visible (dropflags are closed).
{
	if (_data == nil)
	{
		_defaultsSelector = defaultsSelector;
		_data = [options retain];
	
		[self _setOptionsList:[options allKeys]];
	
		if (_table)
		{
			if ([_optionsList count] > 0)
				[_table selectRow:0 byExtendingSelection:NO];
			
			[self tableViewSelectionDidChange:nil];
			
			_window = (window ? window : [_table window]);	//hack upon a hack.
			[[NSNotificationCenter defaultCenter] addObserver: self
													 selector: @selector(_windowWillClose:)
														 name: NSWindowWillCloseNotification
													   object: _window];
		}
	}
	
	return self;
}


- (void) reloadData
{
	[_table reloadData];
}

static id restoreModuleDefaults_map(id object, id key, void* context)
{
	return [object objectAtIndex:kTomatoAgentModuleDefaultsEntryIndex_value];
}

- (void) restoreModuleDefaults:sender;
{
	id		options, keys, key;
			
	options = [[[self _defaultOptions] objectForKey:kTomatoAgentModuleDefaultsKey_map]
				dictionaryMappingObjectsUsingFunction:restoreModuleDefaults_map 
											  context:nil];
	
	//only set keys that are already in _data; addEntriesFromDictionary adds them all.
	keys = [_optionsList objectEnumerator];
	
	while (key = [keys nextObject])
		[_data setObject:[options objectForKey:key] forKey:key];
	
	[self reloadData];
}

//------------------------------------------------------------------------------
//		 download configuration support
//------------------------------------------------------------------------------
//	setModuleConfiguration: and formatterForKey: are temporary, to be replaced
//	by something smarter when I care enough.
//
//	unlike setModuleOptions, moduleConfiguration can be called many times
//	and is used for reflecting the status of a module, rather than defining it.

- setModuleConfiguration:(NSDictionary*)options defaultsSelector:(SEL)defaultsSelector;
{
	_defaultsSelector = defaultsSelector;

	[_data release];
	_data = [options retain];
	
	return self;
}

- (BOOL) keyHasBoolValue:key
{
	id		defaults 	= [self _defaultOptions];
	id		bools 		= [defaults objectForKey:kTomatoAgentModuleDefaultsKey_boolKeys];
	BOOL	result;
	
	result = [bools containsObject:key];
	
	return result;
}

- formatterForKey:key
{
	id		value;
	id		result 		= nil;
	id		defaults 	= [self _defaultOptions];
	id		bools 		= [defaults objectForKey:kTomatoAgentModuleDefaultsKey_boolKeys];

	value = [_data objectForKey:key]; 
	
	/*if ([[NSApp delegate] keyHasBooleanValue:key])*/
	if ([bools containsObject:key])
		result = [[SKBooleanFormatter new] autorelease];
		
	else if (value && [value isKindOfClass:[NSValue class]])
	{
		result = _intFormatter;
		if (![value isIntegerValue])
			result = _realFormatter;
	}
	
	return result;
}

@end


@implementation ModuleOptionsController (Private)
static NSComparisonResult _setOptionsList_sort_localized_keys(id a, id b, void* asdf)
{
	id		a_string = TTAgentLocalizedString_(a, kTomatoAgentStringsTable_moduleDefaults);
	id		b_string = TTAgentLocalizedString_(b, kTomatoAgentStringsTable_moduleDefaults);
	
	return [a_string localizedCompare:b_string];
}
- _setOptionsList:list
{
	
	

	_optionsList = [[list sortedArrayUsingFunction:_setOptionsList_sort_localized_keys
										   context:nil] retain];

	if (_table)
	{
		id	column;
		
		if (column = [_table tableColumnWithIdentifier:kTableColumnIdent_value])
		{
			if ([column respondsToSelector:@selector(setOptionsList:defaultsSelector:)])
				[column setOptionsList:_optionsList defaultsSelector:_defaultsSelector];
		}
	}
	
	return self;
}

- _defaultOptions
{
	if (_defaults == nil)
		_defaults = [TomatoTorrentAgent performSelector:_defaultsSelector];
	
	return _defaults;
}

- (void) _windowWillClose:notif
{
	if ([notif object] == _window)
	{
		[_table setDataSource:nil];
		[_table setDelegate:nil];
		
		[[NSNotificationCenter defaultCenter] removeObserver:self];
	}
}
@end

@implementation ModuleOptionsController (NSTableViewDelegate)
- (void)tableViewSelectionDidChange:(NSNotification *)notification;
{
	if (_optionsList)
	{
		int	selection;
		
		selection = [_table selectedRow];
		if (selection >= 0)
		{
			id		key = [_optionsList objectAtIndex:selection];
	
			[_optionNameText setStringValue:[self _nameOptionsDataWithKey:key]];
			[_optionDescText setStringValue:[self _descOptionsDataWithKey:key]];
		}
		else
		{
			[_optionNameText setStringValue:@""];
			[_optionDescText setStringValue:@""];
		}
	}
}



- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(int)row;
{
	if ([[tableColumn identifier] isEqual:kTableColumnIdent_value])
	{
		id		value 		= [self tableView:tableView objectValueForTableColumn:tableColumn row:row];
		id		formatter 	= nil;
		
		if (value && [value isKindOfClass:[NSValue class]])
		{
			formatter = _intFormatter;
			if (![value isIntegerValue])
				formatter = _realFormatter;
		}
		
		[cell setFormatter:formatter];
	}
}

	//when the formatter fails. default behaviour is to beep and not commit the field editor 
	//the user won't know why they're number isn't being accepted, and the program will seem stuck.
- (BOOL)control:(NSControl *)control didFailToFormatString:(NSString *)string errorDescription:(NSString *)error;
{
	return YES;		//I'll accept it and just turn it back into the right format. It's a real vs. int problem.
}

@end

@implementation ModuleOptionsController (NSTableDataSource)
- (int)numberOfRowsInTableView:(NSTableView *)tableView;
{
	int	result = 0;

	if (_optionsList)
		result = [_optionsList count];
	
	return result;
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row;
{
	id		result = nil, 
			selectorName;
	SEL		selector;
	
	if (_optionsList)
	{
		//my heart didn't want to do a bunch of ifs.
		selectorName = [[NSString alloc] initWithFormat:@"_%@OptionsDataWithKey:", [tableColumn identifier]];
		selector = NSSelectorFromString(selectorName);
		[selectorName release];
		
		if ([self respondsToSelector:selector])
		{
			result = [self performSelector:selector withObject:[_optionsList objectAtIndex:row]];
		}
	}
	
	return result;
}

- (void)tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(int)row;
{
	if (object != nil)
	{
		if ([[tableColumn identifier] isEqual:kTableColumnIdent_value])
		{
			id		key 			= [_optionsList objectAtIndex:row];
			id		old 			= [_data objectForKey:key];			//original triple (array)
			id		new				= object;

			//if the original is an integer, make sure we set an integer. Only important because
			//when the new value is entered, it /will/ be a float, and the user can now enter floats in the field.
			//the back-end actually doesn't care, it gets stuff as strings and the coerces it to the right type
			//(using the same technique here: checking against the default value in BitTorrent.*.defaults).
			
			if ([old isKindOfClass:[NSValue class]])
			{
				if ([old isIntegerValue])
					new = [NSNumber numberWithInt:[new intValue]];	//nsnumber & nsstring respond.
				else
					new = [NSNumber numberWithDouble:[new doubleValue]];
			}
			
			[_data setObject:new forKey:key];
		}
	}
}

- _nameOptionsDataWithKey:key
{
	//key also happens to be the value of name.
	return TTAgentLocalizedString_(key, kTomatoAgentStringsTable_moduleDefaults);
}

- _valueOptionsDataWithKey:key
{
	return [_data objectForKey:key];
}

- _descOptionsDataWithKey:key
{
	id		entry, result = nil;
	id		defaults = [[self _defaultOptions] objectForKey:kTomatoAgentModuleDefaultsKey_map];
	id		defaultDesc;
	id		value;
	
	if (entry = [defaults objectForKey:key])
	{
		//that's an array, we want the second item.
		result = [entry objectAtIndex:kTomatoAgentModuleDefaultsEntryIndex_description];

		//localize it.
		result = TTAgentLocalizedString_(result, kTomatoAgentStringsTable_moduleDefaults);
		
		defaultDesc = SKLocalizedString(@"(Default: %@)", @"default value description for option (value)");
		
		value = [entry objectAtIndex:kTomatoAgentModuleDefaultsEntryIndex_value];
		value = [[NSApp delegate] stringForObjectValue:value forKey:key];
		
		defaultDesc = [NSString stringWithFormat:defaultDesc, value];
		
		//append the default value
		result = [NSString stringWithFormat:@"%@\n%@", result, defaultDesc];
	}
	
	return result;
}

@end

@implementation ModuleOptionsValueTableColumn
- (void) dealloc
{
	[_checkboxCell release];
	[_list release];
	[super dealloc];
}

- (void) awakeFromNib
{
	_checkboxCell = [NSButtonCell new];
	[_checkboxCell setButtonType:NSSwitchButton];
	[_checkboxCell setImagePosition:NSImageLeft];
	[_checkboxCell setTitle:@""];
	[_checkboxCell setControlSize:NSSmallControlSize];
}

- setOptionsList:list defaultsSelector:(SEL)selector;
{
	[_list release];
	_list = [list retain];
	_sel = selector;
	return self;
}

- _defaultOptions
{
	if (_defaults == nil && _sel != nil)	//don't have list yet; initial view draw.
		_defaults = [TomatoTorrentAgent performSelector:_sel];
	
	return _defaults;
}

- dataCellForRow:(int)row
{
	id	result = nil;
	id	bools = [[self _defaultOptions] objectForKey:kTomatoAgentModuleDefaultsKey_boolKeys];

	if (row >= 0 && _list != nil)
	{
		//if ([[NSApp delegate] keyHasBooleanValue:[_list objectAtIndex:row]])
		if ([bools containsObject:[_list objectAtIndex:row]])
			result = _checkboxCell;
	}
	
	if (result == nil)
		result = [super dataCellForRow:row];
	
	return result;
}

@end

@implementation NSValue (LazySarwat)
- (BOOL) isIntegerValue;
{
	const char	*type = [self objCType];
	
	return !(strcmp(type, @encode(float)) == 0 || strcmp(type, @encode(double)) == 0);
}
@end
