//===========================================================================
//		• PreferencesController
//===========================================================================

#import "PreferencesController.h"

#import "TomatoTorrentAgent.h"

//support controllers
@interface ShareRatioPrefsController : NSObject
{
    IBOutlet id __downloadExampleText;
    IBOutlet id __ratioText;
    IBOutlet id __slider;
    IBOutlet id __uploadExampleText;
}
- (IBAction)shareRatioSliderAction:(id)sender;
- (IBAction)shareRatioTextFieldAction:(id)sender;
@end

@interface SeedingTimeoutPrefsController : NSObject
{
    IBOutlet id __stepper;
    IBOutlet id __timeoutField;
    IBOutlet id __unitToggleButton;
}
- (IBAction)timeUnitsToggleAction:(id)sender;
@end

@interface UploadLimitPrefsController: NSObject
{ id __uploadLimitTextField; }
@end

//defaults used by prefscontroller
NSString* const kTomatoAutoStopTorrentsMethodDefault = @"kTomatoAutoStopTorrentsMethodDefault";
NSString* const kTomatoAutoStopTorrentsDefault = @"kTomatoAutoStopTorrentsDefault";
NSString* const kTomatoAutoStopTorrentUploadLimitDefault = @"kTomatoAutoStopTorrentUploadLimitDefault";	//megabytes
NSString* const kTomatoAutoStopTorrentShareRatioDefault = @"kTomatoAutoStopTorrentShareRatioDefault";	//0-1.0
NSString* const kTomatoAutoStopTorrentTimeLimitDefault = @"kTomatoAutoStopTorrentTimeLimitDefault";	//minutes
NSString* const kTomatoSecondaryUploadRateKBsDefault = @"kTomatoSecondaryUploadRateKBsDefault";
NSString* const kTomatoChangeMaxUploadRateOnCompletionDefault = @"kTomatoChangeMaxUploadRateOnCompletionDefault";

//defaults used by thingy
NSString* const kTomatoAutoStopShareRatioTimeLimitShowingHoursDefault = @"kTomatoAutoStopShareRatioTimeLimitShowingHoursDefault";

//notifications
NSString* const kTomatoAutoStopPrefsDidChangeNotification = @"kTomatoAutoStopPrefsDidChangeNotification";

#define kMaximumShareRatio 5.0

static id gPrefsController = nil;

//===========================================================================
//		• @implementation PreferencesController
//===========================================================================

@implementation PreferencesController

+ (void) initialize
{
	static bool foo = false; if (foo) return; foo = true;
	
	[[NSUserDefaults standardUserDefaults] registerDefaults:
		[NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithInt:kAutoStopTorrentMethod_shareRatio],
			kTomatoAutoStopTorrentsMethodDefault,
			
			[NSNumber numberWithBool:NO],
			kTomatoAutoStopTorrentsDefault,
			
			[NSNumber numberWithInt:42],
			kTomatoAutoStopTorrentUploadLimitDefault,
			
			[NSNumber numberWithFloat:1.5],
			kTomatoAutoStopTorrentShareRatioDefault,
			
			[NSNumber numberWithInt:15],
			kTomatoAutoStopTorrentTimeLimitDefault,
			
			[NSNumber numberWithInt:7],
			kTomatoSecondaryUploadRateKBsDefault,
			
			[NSNumber numberWithBool:NO],
			kTomatoChangeMaxUploadRateOnCompletionDefault,
			
			nil]];
}

+ (BOOL) doAutoStopTomatoTorrentDownloads;
{
	return [[NSUserDefaults standardUserDefaults] boolForKey:kTomatoAutoStopTorrentsDefault];
}

+ (TomatoTorrentDownloadAutoStopMethod) tomatoDownloadAutoStopMethod;
{
	return (TomatoTorrentDownloadAutoStopMethod)[[NSUserDefaults standardUserDefaults] integerForKey:kTomatoAutoStopTorrentsMethodDefault];
}

+ (float) tomatoTorrentTargetDownloadShareRatio;
{
	float		result = [[NSUserDefaults standardUserDefaults] floatForKey:kTomatoAutoStopTorrentShareRatioDefault];
	
	if (result < 0) result = 0;
	
	return result;
}

+ (unsigned) seedingUploadLimitAsMegabytes
{
	int		result = [[NSUserDefaults standardUserDefaults] integerForKey:kTomatoAutoStopTorrentUploadLimitDefault];
	
	if (result < 0) result = 0;
	
	return result;
}

+ (NSTimeInterval) seedingAutoStopTimeInterval;
{
	int				minutes = [[NSUserDefaults standardUserDefaults] integerForKey:kTomatoAutoStopTorrentTimeLimitDefault];
	NSTimeInterval	result = 0;
	
	if (minutes >= 0)
		result = minutes * 60;
	
	return result;
}

+ (unsigned) secondaryUploadRateKBs;
{
	int		result = [[NSUserDefaults standardUserDefaults] integerForKey:kTomatoSecondaryUploadRateKBsDefault];
	
	if (result < 0)
		result = 0;
	
	return result;
}

+ (BOOL) usesSecondaryUploadRateForCompletedTorrents
{
	return [[NSUserDefaults standardUserDefaults] boolForKey:kTomatoChangeMaxUploadRateOnCompletionDefault];
}

//this is a performance hack.
static bool gCachedAutoStopByShareRatio = NO;
static bool	gPerformanceHackValid = NO;
+ (BOOL) doAutoStopTomatoTorrentDownloadsByShareRatio
{
	BOOL	result = gCachedAutoStopByShareRatio;
	
	if (!gPerformanceHackValid)
	{
		gCachedAutoStopByShareRatio = 
		([self doAutoStopTomatoTorrentDownloads] && ([self tomatoDownloadAutoStopMethod] == kAutoStopTorrentMethod_shareRatio));
		
		gPerformanceHackValid = YES;
	}
	
	return result;
}

+ (void) postDefaultsChangedNotification;
{
	gPerformanceHackValid = NO;
	
	[[NSNotificationCenter defaultCenter] postNotificationName: kTomatoAutoStopPrefsDidChangeNotification
														object: nil];
}

+ (void) noteTorrentDownloadOptionsDefaultsChanged
{
	gPerformanceHackValid = NO;
	
	if (gPrefsController)
		[gPrefsController update];
}

+ sharedInstance
{
	if (gPrefsController == nil)
		gPrefsController = [self new];
	
	return gPrefsController;
}

- init
{
	return [super initWithWindowNibName:@"Preferences"];
}

//---------------------------------------------------------------------------
//		• - update
//---------------------------------------------------------------------------
//	Update the 2 edit fields and 1 checkbox. Called by -windowDidLoad and
//	+noteTorrentDownloadOptionsDefaultsChanged.

- (void) update
{
	id		defaults, downloadOptions;
	int		rate;
	
	defaults = [NSUserDefaults standardUserDefaults];
	downloadOptions = [defaults objectForKey:kUserDefault_TomatoAgent_downloadOptions];
	
	if (downloadOptions)
	{
		rate = [[downloadOptions objectForKey:kTomatoAgentKey_downloadParameter_maxUploadRate] intValue];
		if (rate >= 0) [__maxUploadRateField setIntValue:rate];
	}
	
	rate = [defaults integerForKey:kTomatoSecondaryUploadRateKBsDefault];
	if (rate >= 0) [__seedingMaxUploadRateField setIntValue:rate];
	
	[__seedingMaxUploadRateCheckbox setState:[defaults boolForKey:kTomatoChangeMaxUploadRateOnCompletionDefault]];
	
	[__autoCloseRadioMatrix selectCellWithTag:[defaults integerForKey:kTomatoAutoStopTorrentsMethodDefault]];
	[__autoCloseTabView selectTabViewItemAtIndex:[defaults integerForKey:kTomatoAutoStopTorrentsMethodDefault]];
	[__autoCloseCheckbox setState:[defaults boolForKey:kTomatoAutoStopTorrentsDefault]];
}

- (void) windowDidLoad
{
	[self update];
	[[self window] center];
}

- (IBAction)autoCloseTorrentsCheckboxAction:(id)sender
{
	[[NSUserDefaults standardUserDefaults] setBool:[sender state] forKey:kTomatoAutoStopTorrentsDefault];
	[PreferencesController postDefaultsChangedNotification];
}

- (IBAction)autoCloseTorrentsChoiceAction:(id)sender
{
	int	foo = [[sender selectedCell] tag];
	[__autoCloseTabView selectTabViewItemAtIndex:foo];
	[[NSUserDefaults standardUserDefaults] setInteger:foo forKey:kTomatoAutoStopTorrentsMethodDefault];
	
	[PreferencesController postDefaultsChangedNotification];
}

- (IBAction)changeMaxUploadRateCheckboxAction:(id)sender
{
	[[NSUserDefaults standardUserDefaults] setBool:[sender state] forKey:kTomatoChangeMaxUploadRateOnCompletionDefault];
	[PreferencesController postDefaultsChangedNotification];
}

- (IBAction)defaultUploadRateTextAction:(id)sender
{
	id		currentDownloadOptions, defaults;
	int		thing = [sender intValue];
	
	if (thing >= 0)
	{
		//<sigh>.
		defaults = [NSUserDefaults standardUserDefaults];
		currentDownloadOptions = [defaults objectForKey:kUserDefault_TomatoAgent_downloadOptions];
		if (currentDownloadOptions) currentDownloadOptions = [NSMutableDictionary dictionaryWithDictionary:currentDownloadOptions];
		else currentDownloadOptions = [NSMutableDictionary dictionary];
		[currentDownloadOptions setObject:[NSNumber numberWithInt:thing] forKey:kTomatoAgentKey_downloadParameter_maxUploadRate];
		[defaults setObject:currentDownloadOptions forKey:kUserDefault_TomatoAgent_downloadOptions];
		
		[PreferencesController postDefaultsChangedNotification];
	}
}

- (IBAction)secondaryUploadRateTextAction:(id)sender
{
	[[NSUserDefaults standardUserDefaults] setInteger:[sender intValue] forKey:kTomatoSecondaryUploadRateKBsDefault];
	[PreferencesController postDefaultsChangedNotification];
}
@end

@implementation ShareRatioPrefsController

- (void) updateRatioControlsWithRatio:(float)ratio
{
	[__ratioText setIntValue:ratio * 100];
	[__slider setFloatValue:ratio];
	
	#define kExampleDownloadSize 50
	
	[__downloadExampleText setStringValue:
		[NSString stringWithFormat:NSLocalizedString(@"%d MB download", @"ratio dowload example text"), kExampleDownloadSize]];
	
	[__uploadExampleText setStringValue:
		[NSString stringWithFormat:NSLocalizedString(@"%d MB upload", @"ratio uplaod example text"), (int)(kExampleDownloadSize * ratio)]];
}

- (void) awakeFromNib
{
	float		ratio = [[NSUserDefaults standardUserDefaults] floatForKey:kTomatoAutoStopTorrentShareRatioDefault];
	
	//this is stupid, but whatever.
	[[PreferencesController sharedInstance] window];

	[self updateRatioControlsWithRatio:ratio];
}

- (IBAction)shareRatioSliderAction:(id)sender;
{
	float		ratio = [sender floatValue];
	
	if (ratio >= 0)
	{
		[[NSUserDefaults standardUserDefaults] setFloat:ratio forKey:kTomatoAutoStopTorrentShareRatioDefault];
		[PreferencesController postDefaultsChangedNotification];

		[self updateRatioControlsWithRatio:ratio];
	}
}

- (IBAction)shareRatioTextFieldAction:(id)sender;
{
	int		ratioPercent = [sender intValue];
	
	if (ratioPercent >= 0)
	{
		float		ratio = ratioPercent/100.0;
		
		if (ratio > kMaximumShareRatio) ratio = kMaximumShareRatio;
		
		[[NSUserDefaults standardUserDefaults] setFloat:ratio forKey:kTomatoAutoStopTorrentShareRatioDefault];
		[PreferencesController postDefaultsChangedNotification];

		[self updateRatioControlsWithRatio:ratio];
	}
}

@end

@implementation SeedingTimeoutPrefsController : NSObject
- (void) awakeFromNib
{
	id		defaults;
	int		numMinutes;
	bool	showHours;
	
	//this is stupid, but whatever.
	[[PreferencesController sharedInstance] window];
	
	defaults = [NSUserDefaults standardUserDefaults];
	numMinutes = [defaults integerForKey:kTomatoAutoStopTorrentTimeLimitDefault];
	showHours = [defaults boolForKey:kTomatoAutoStopShareRatioTimeLimitShowingHoursDefault];	//initially NO.
	
	if (NSClassFromString(@"NSTokenField") == nil)
	{
		//we don't have 10.4,
		[__unitToggleButton setBezelStyle:NSShadowlessSquareBezelStyle];
		[__unitToggleButton sizeToFit];
	}
	
	[__unitToggleButton setState:showHours];
	if (showHours)
	{
		[__unitToggleButton setState:NSOnState];
		[__timeoutField setIntValue:numMinutes / 60];
	}
	else
		[__timeoutField setIntValue:numMinutes];
}

- (void) controlTextDidChange:(NSNotification*)notif
{
	int		currentValue = [__timeoutField intValue];
	
	if ([__unitToggleButton state] == NSOnState)
	{
		currentValue *= 60;
	}
	
	[[NSUserDefaults standardUserDefaults] setInteger:currentValue forKey:kTomatoAutoStopTorrentTimeLimitDefault];
	[PreferencesController postDefaultsChangedNotification];
}

- (IBAction)timeUnitsToggleAction:(id)sender;
{
	[[NSUserDefaults standardUserDefaults] setBool:[sender state] forKey:kTomatoAutoStopShareRatioTimeLimitShowingHoursDefault];
	[self controlTextDidChange:nil];
}

@end

@implementation UploadLimitPrefsController
- (void) awakeFromNib
{
	//this is stupid, but whatever.
	[[PreferencesController sharedInstance] window];

	[__uploadLimitTextField setIntValue:[[NSUserDefaults standardUserDefaults] integerForKey:kTomatoAutoStopTorrentUploadLimitDefault]];
}

- (void) uploadLimitTextAction:sender
{
	int		thing = [sender intValue];
	
	if (thing >= 0)
	{
		[[NSUserDefaults standardUserDefaults] setInteger:thing forKey:kTomatoAutoStopTorrentUploadLimitDefault];
		[PreferencesController postDefaultsChangedNotification];
	}
}
@end

@interface NonUselessTextField: NSTextField @end

@implementation NonUselessTextField
- (void)takeIntValueFrom:(id)sender;
{
	[super takeIntValueFrom:sender];
	[self sendAction:[self action] to:[self target]];
}
@end

@interface TomatoView: NSView {id __tomatoImage;} @end
@implementation TomatoView: NSView
- (void) awakeFromNib
{
	__tomatoImage = [[NSImage imageNamed:@"AppIcon"] retain];
}

- (BOOL) isOpaque
{
	return NO;
}

- (void) drawRect:(NSRect)update
{
	NSRect		imageRect,
				bounds,
				targetRect;
	NSSize		imageSize;
	
	bounds = [self bounds];
	imageSize = [__tomatoImage size];
	imageRect = targetRect = NSMakeRect(0, 0, imageSize.width, imageSize.height);
	
	targetRect.origin.x = NSMidX(bounds) - NSWidth(imageRect)/2;
	targetRect.origin.y = NSMidY(bounds) - NSHeight(imageRect)/2;
	
	[__tomatoImage drawInRect: targetRect
					 fromRect: imageRect
					operation: NSCompositeSourceOver
					 fraction: 1];
}
@end
