//==============================================================================
//		TorrentFaultFormatter.m					TomatoTorrent
//==============================================================================
//  Created by Sarwat Khan on Fri Jun 06 2003.
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khans name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import "TorrentFaultFormatter.h"

#import "SKContainerFormatter.h"
#import "TomatoTorrentAgent.h"

@implementation TorrentFaultFormatter
- (void) dealloc
{
	[_downloadFailure release];
	[_agent release];

	[super dealloc];
}


- setDownloadFailureResponse:response fromAgent:agent;	//agent may be nil.
{
	[_downloadFailure release];
	_downloadFailure = [response retain];

	[_agent release];
	_agent = [agent retain];

	return self;
}

- _formatBug:failure
{
	id		ite, obj;
	id		result = [NSMutableString string];
	int		count = 0;
	id		string;
	id		locale = [NSDictionary dictionary];
	
	ite = [failure objectEnumerator];
	while (obj = [ite nextObject])
	{
		string = [SKContainerFormatter stringForObjectValue:obj locale:locale];
		[result appendFormat:@"%d: %@\n", 
			++count, 
			TTAgentLocalizedString_(string, kTomatoAgentStringsTable_responseFaults)];
	}
	
	return result;
}

- _formatDownloadFailure:failure
{
	//find an object that contains an array of size two.
	id		ite, obj;
	id		result = nil;
	
	ite = [failure objectEnumerator];
	while (obj = [ite nextObject])
	{
		if ([obj respondsToSelector:@selector(count)]
			&& [obj count] == 2)
		{
			//assume that this is an exception (class, desc).
			//Only print the desc, and if the desc contains a url we toy with it.
			id			desc = [obj objectAtIndex:1];
			id			url = nil;// = [_agent sourceURL];
			id			local;
			
			NSRange		range;
			
			if (url)
			{
				range = [desc rangeOfString:url];
				if (range.location != NSNotFound)
				{
					int		urlIndex = range.location;
					
					desc = [NSMutableString stringWithString:desc];
					[desc deleteCharactersInRange:range];
					
					local = SKLocalizedStringWithDefaultValue(
						desc, 
						kTomatoAgentStringsTable_responseFaults, 
						TTAgentBundle_(), 
						desc,
						nil);
					
					//see if it was localized (didn't get desc back).
					range = [local rangeOfString:@"%@"];
					if (range.location == NSNotFound)
					{
						desc = [NSMutableString stringWithString:local];
						[desc insertString:url atIndex:urlIndex];
					}
					else
					{
						desc = [NSString stringWithFormat:local, url];
					}
				}
			}
			
			result = desc;
			break;
		}
	}
	
	if (result == nil)
		result = [self _formatBug:failure];
	
	return result;
}

- (NSString *)stringForObjectValue:(id)failure;
{
	id result;
	
	if (_downloadFailure)
		result = [self _formatDownloadFailure:failure];
	else if ([failure respondsToSelector:@selector(objectEnumerator)])
		result = [self _formatBug:failure];
	else
		result = [failure description];
		
	return result;
}

- (BOOL)getObjectValue:(id *)obj forString:(NSString *)string errorDescription:(NSString **)error;
{
	return NO;
}

@end
