//==============================================================================
//		 TrackerDocumentController.m
//==============================================================================
//  Copyright (c) 2003, Sarwat Khan. All rights reserved.
//  
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//          
//      1. Redistributions of source code must retain the above copyright
//         notice, this list of conditions and the following disclaimer.
//          
//      2. Redistributions in binary form must reproduce the above
//         copyright notice, this list of conditions and the following disclaimer
//         in the documentation and/or other materials provided with the
//         distribution.
//          
//      3. Neither Sarwat Khans name nor the names of any contributors to
//         the Tomato Torrent Software may be used to endorse or promote
//         products derived from this software without specific prior written
//         permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

//	I wrote this really, really quickly. I'm not sure if it all works.
//	I'm not even sure if anyone will use it.


#import "TrackerDocumentController.h"

#import "TrackerWindowController.h"
#import "TrackerDocument.h"

#import "ModuleOptionsController.h"
#import "NSWindow+WindowContentSize.h"
#import "NSBox+DynamicContent.h"

#import "PrimaryIPv4Address.h"

#include <unistd.h>

#define kThreadProvidedData		@"ThreadProvidedData"


@interface TrackerDocumentController (Private)
- _startTrackingAgent:logPath;
- _stopTrackingAgent;
- _updateStartButton;
- _trackerURLWithPortNumber:(int)port;
- _setWindowSizeLimitMin:(NSSize)size max:(NSSize)size;

- (void) _ipUpdated:notif;
- (void)_chooseAllowedDirPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
@end


@implementation TrackerDocumentController

- (void) dealloc
{
	[self _stopTrackingAgent];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[_saveEmptyView release];
	[_hostID release];
	
//	[_tableController release];
	[super dealloc];
}

- (void) awakeFromNib
{
	NSRect		frame, emptyFrame;
	
	_hostID = [PrimaryIPv4Address new];
	
	frame = [_runView frame];
	_initialRunViewSize = frame.size;
	
	frame = [_editView frame];
	_initialEditViewSize = frame.size;

	_saveEmptyView = [[_advancedOptionsBox contentView] retain];
	emptyFrame = [_saveEmptyView frame];

	_expandedEditViewSize.width = _initialEditViewSize.width;
	_expandedEditViewSize.height = _initialEditViewSize.height + frame.size.height - emptyFrame.size.height;
	
}

+ makeTrackerDocumentWindowController
{
	return [TrackerWindowController new];
}

- windowController
{
	return _windowController;
}

- window
{
	return [_windowController window];
}

- document
{
	return [_windowController document];
}

- documentIsSpecified:(BOOL)is;
{
	[self _updateStartButton];
	return self;
}

//------------------------------------------------------------------------------
//		 reloadData
//------------------------------------------------------------------------------
//	Also validates the data (modifies the model as it draws).

- (void) reloadData
{
	id		object, allowedDirPath;
	int		lastPort, port;
	int		state;
	BOOL	isDir = NO;
	
	if ([[self document] trackerIsRunning] == NO)
	{
		
		//listen port
		lastPort = [_listenPortText intValue];
		port = [[self document] trackerPort];	//current value.
		
		if (lastPort != port)
		{
			//update the tracker url
			[_trackerURLText setStringValue:[self _trackerURLWithPortNumber:port]];
			
			[_listenPortText setIntValue:port];
		}
		
		//allowed dir button & text.
		state = NSOffState;
		allowedDirPath = @"";
		
		if (object = [[self document] trackerAllowedDirPath])
		{
			if ([object isAbsolutePath]
				&& [[NSFileManager defaultManager] fileExistsAtPath:object isDirectory:&isDir] == YES && isDir == YES)
			{
				state = NSOnState;
				allowedDirPath = [object stringByAbbreviatingWithTildeInPath];
			}
			else
				[[self document] setTrackerAllowedDirPath:nil];
			
		}
		
		[_allowedDirText setStringValue:allowedDirPath];
		[_allowedDirButtton setState:state];
		
		//advanced options.
		[_tableController reloadData];
		
		[self _updateStartButton];
	}
}

- showTrackerProgress:logFilePath;
{
	[self _startTrackingAgent:logFilePath];
	return self;
}

- stopShowingTrackerProgress;
{
	[self _stopTrackingAgent];
	return self;
}

- (IBAction)chooseAllowedDir:(id)sender
{
	id		panel = [NSOpenPanel openPanel];
	id		object,
			dir = NSHomeDirectory();
	BOOL	isDir;
	
	if (object = [[self document] trackerAllowedDirPath])
	{
		if ([(NSString*)object length] > 0
			&& [[NSFileManager defaultManager] fileExistsAtPath:object isDirectory:&isDir] == YES && isDir == YES)
			dir = object;
	}
	
	[panel setCanChooseDirectories:YES];
	[panel setCanChooseFiles:NO];
	[panel setAllowsMultipleSelection:NO];
	
	[panel beginSheetForDirectory:dir 
							 file:nil
							types:nil 
				   modalForWindow:[self window]
				    modalDelegate:self
				   didEndSelector:@selector(_chooseAllowedDirPanelDidEnd:returnCode:contextInfo:)
				      contextInfo:nil];
}

- (IBAction)listenPortEdited:(id)sender
{
	id		value;
	
	if (value = [sender objectValue])
	{
		[[self document] setTrackerPort:[value intValue]];
		[self reloadData];
	}
}

- (IBAction)restoreOptionsDefaults:(id)sender
{
	[[self document] restoreDefaultSettings];
}

- (IBAction)setAllowedDirOn:(id)sender
{
	NSString	*dir;
	BOOL		isDir = NO;

	if ([sender state] == NSOnState)
	{
		if (dir = [[self document] trackerAllowedDirPath])
		{
			if ([dir isAbsolutePath] == NO ||
				[[NSFileManager defaultManager] fileExistsAtPath:dir isDirectory:&isDir] == NO || isDir == NO)
			{
				[self chooseAllowedDir:nil];
			}
		}
		else
			[self chooseAllowedDir:nil];
	}
	else
	{
		[[self document] setTrackerAllowedDirPath:nil];
		[self reloadData];
	}
}

- (IBAction)saveTrackerAs:(id)sender
{
	[[self document] saveDocumentAs:sender];
}

- (IBAction)startOrSaveTracker:(id)sender
{
	if ([[self document] fileName])
	{
		[[self document] saveDocument:sender];
		[[self document] startTracker:sender];
	}
	else
	{
		[[self document] saveDocument:sender];		
	}
}

- (IBAction)stopRunningTracker:(id)sender
{
	[[self document] stopTracker:sender];
}

- (IBAction)toggleAdvancedOptions:(id)sender
{
	id			view = _saveEmptyView;
	BOOL		showOptions = [sender state] == NSOnState;

	[[self window] setShowsResizeIndicator:showOptions];

	if (showOptions) 
	{
		[_tableController setModuleOptions:[[self document] settings] 
						  defaultsSelector:@selector(defaultTrackerOptions)
						  controllingTableForWindow:[self window]];

		view = _advancedOptionsView;
		[self _setWindowSizeLimitMin:_expandedEditViewSize max:NSMakeSize(832, 624)];
	}
	else
		[self _setWindowSizeLimitMin:_initialEditViewSize max:NSMakeSize(832, _initialEditViewSize.height)];
	
	//[_advancedOptionsBox switchContentViewTo:view beforeResizing:!showOptions];
	[_advancedOptionsBox switchContentViewTo:view showingViewDuringResize:_saveEmptyView];

	if (showOptions) [self reloadData];
}

- setupWindowForDocumentState
{
	id	view, log;

	//install the view
	view = _editView;
	if ([[self document] trackerIsRunning]) 
	{
		view = _runView;
		[self _setWindowSizeLimitMin:_initialRunViewSize max:NSZeroSize];
	}

	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(windowWillClose:)
												 name:NSWindowWillCloseNotification
											   object:[self window]];
	
	if (view == _editView)
	{
		[_windowController showView:view];
		[self _setWindowSizeLimitMin:_initialEditViewSize max:NSMakeSize(800, _initialEditViewSize.height)];
		
		[self _updateStartButton];
		[self reloadData];
		
		/*if (_tableController == nil)
			_tableController = [[ModuleOptionsController alloc]
				initWithContentView:_advancedOptionsView
					  moduleOptions:[[self document] settings]
						   defaults:[TomatoTorrentAgent defaultTrackerOptions]];
		*/
	}
	else if (log = [[self document] trackerLogFilePath])
	{
		[self showTrackerProgress:log];
	}
	else
	{
		[_windowController showView:view];
		[self stopShowingTrackerProgress];
	}
	
	return self;
}

- (void) windowWillClose:notif
{
	[[self document] stopTracker:nil];	//this is async, so let's stop ourselves.
	[self _stopTrackingAgent];
}

- (void)controlTextDidChange:(NSNotification *)notif;
{
	int		port = [_listenPortText intValue];

	if ([notif object] == _listenPortText)
	{
		[_listenPortText validateEditing];
		
		if (port > 0 && [_listenPortText intValue] == port)
		{
			[[self document] setTrackerPort:port];
			[_trackerURLText setStringValue:[self _trackerURLWithPortNumber:port]];
		}
		else
		{
			[_listenPortText setIntValue:[[self document] trackerPort]];
		}
	}
}

@end

@implementation TrackerDocumentController (Private)
- _startTrackingAgent:path;
{
	id		readPipe;
	
	//get the tracker's file and spawn a tail -f process to track it.
	if (_task)
		[self _stopTrackingAgent];
	
	//[NSThread detachNewThreadSelector:@selector(_sendLogData:) toTarget:self withObject:path];
	
	readPipe = [NSPipe pipe];
	_fileHandle = [[readPipe fileHandleForReading] retain];
	
	_task = [NSTask new];

	//run tail -f on the log file.
	[_task setLaunchPath:@"/usr/bin/tail"];
	[_task setArguments:[NSArray arrayWithObjects:@"-f", path, nil]];
	
	[_task setStandardOutput:readPipe];
	[_task setStandardError:readPipe];
	
	[_task launch];
	
	if ([_task isRunning])
	{
		//gui setup.
		[_stopButton setEnabled:YES];
		
		//show date
		[_trackerStartDateText setStringValue:
			[NSString stringWithFormat:
				SKLocalizedString(@"Tracker Started at %@", @"Tracker Started at (date))"),
				[NSCalendarDate date]]];
		
		//show log path
		[_loggingToText setStringValue:path];
		
		//I decided to get a little fancy.
		{
			id		commandString,
					newAttribs,
					font, attribString;
			
			commandString = [NSString stringWithFormat:@"%@\n\n/usr/bin/tail -f %@", 
				[self _trackerURLWithPortNumber:[[self document] trackerPort]], path];

			font = [NSFont fontWithName:@"HelveticaNeue-Italic" size:10];
			
			__fontAttribs = [[[_trackerOutput textStorage] attributesAtIndex:0 effectiveRange:nil] retain];
			newAttribs = [NSDictionary dictionaryWithObject:font forKey:NSFontAttributeName];

			attribString = [[[NSAttributedString alloc] initWithString:commandString attributes:newAttribs] autorelease];
			[[_trackerOutput textStorage] replaceCharactersInRange:NSMakeRange(0, [[_trackerOutput textStorage] length])
											  withAttributedString:attribString];
											  
			attribString = [[[NSAttributedString alloc] initWithString:@"\n" attributes:__fontAttribs] autorelease];
			[[_trackerOutput textStorage] appendAttributedString:attribString];
		}
		
		[_windowController showView:_runView];
		
		[self _setWindowSizeLimitMin:_initialRunViewSize max:NSMakeSize(800, FLT_MAX)];
		
		//start a read operation. the downside to doing it this way is the number of threads spawned.
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(_threadProvidedData:)
													 name:NSFileHandleDataAvailableNotification
												   object:_fileHandle];
													
		
		[_fileHandle waitForDataInBackgroundAndNotify];
	}
	else
	{
		[_task release]; 
		_task = nil;
		
		[_fileHandle release];
		_fileHandle = nil;
		
		NSLog(@"couldn't run /usr/bin/tail -f %@", path);
	}
	
	return self;
}

- _stopTrackingAgent;
{
	//kill the task.
	if (_task)
	{
		if ([_task isRunning])
			[_task terminate];
		
		[_task release];
		_task = nil;

		[_fileHandle release];
		_fileHandle = nil;
		
		[_trackerStartDateText setStringValue:
			[NSString stringWithFormat:
				SKLocalizedString(@"Tracker Stopped at %@", @"Tracker Stopped at (date))"),
				[NSCalendarDate date]]];

		if ([[self document] trackerLogFilePath])
		{
			//ok. I want the last four lines of the log displayed. It's bugging me.
			NSTask*	task;
			id		pipe, data, string;
			
			task = [[NSTask new] autorelease];
			pipe = [NSPipe pipe];
			
			[task setLaunchPath:@"/usr/bin/tail"];
			[task setArguments:[NSArray arrayWithObjects:@"-n", @"4", [[self document] trackerLogFilePath], nil]];
			[task setStandardOutput:pipe];
			[task launch];
			[task waitUntilExit];
			
			NS_DURING
				data = [[pipe fileHandleForReading] availableData];
				string = [[[NSString alloc] initWithData:data encoding:NSNEXTSTEPStringEncoding] autorelease];
				string = [[[NSAttributedString alloc] initWithString:string attributes:__fontAttribs] autorelease];

				[[_trackerOutput textStorage] appendAttributedString:string];
				
			NS_HANDLER
				NSLog(@"whoops.");
				
			NS_ENDHANDLER
			
		}
	}
	
	[_stopButton setEnabled:NO];
	
	//show date
	[_trackerStartDateText setStringValue:
		[NSString stringWithFormat:
			SKLocalizedString(@"Tracker Stopped at %@", @"Tracker Stopped at (date))"),
			[NSCalendarDate date]]];
	
	
	return self;
}

- _updateStartButton;
{
	if ([[self document] fileName])
	{
		[_startSaveButton setTitle:SKLocalizedString(@"Start Tracker", @"tracker start button title")];
		[[self window] setDefaultButtonCell:[_startSaveButton cell]];
	}
	else
	{
		[_startSaveButton setTitle:SKLocalizedString(@"Save Tracker", @"tracker save button title")];
	}
	
	return self;
}

- _trackerURLWithPortNumber:(int)port
{
	char			hostname[256];
	id				addr, 
					name = @"";
	
	hostname[0] = hostname[255] = 0;	//make sure it's terminated.
	gethostname(hostname, 255);			//one less.
	
	addr = [NSString stringWithFormat:@"http://%@:%d/announce", 
				[_hostID IPv4String], port];

	if (strlen(hostname) > 0)
		name = [NSString stringWithFormat:@"http://%s:%d/announce", hostname, port];
	
	return [NSString stringWithFormat:@"%@\n%@", addr, name];
}

- (void) _ipUpdated:notif;
{
	[self reloadData];
}

- (void)_chooseAllowedDirPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
{
	if (returnCode == NSOKButton)
	{
		[[self document] setTrackerAllowedDirPath:[[sheet filenames] lastObject]];
		[self reloadData];
	}
}

- (void)_threadProvidedData:notif
{
	id		file;
	
	if (file = [notif object])
	{
		id		string;
		id		attrString;
		id		data;
		
		data = [file availableData];
		
		string = [[NSString alloc] initWithData:data encoding:NSNEXTSTEPStringEncoding];
		attrString = [[NSAttributedString alloc] initWithString:string attributes:__fontAttribs];

		[[_trackerOutput textStorage] appendAttributedString:attrString];
		[_trackerOutput moveToEndOfDocument:nil];
		
		[string release];
		[attrString release];
		
		[_fileHandle waitForDataInBackgroundAndNotify];
	}
}

- _setWindowSizeLimitMin:(NSSize)size max:(NSSize)max
{
	[[self window] setMinimumContentSize:size maximumSize:max];
	return self;
}

@end
